from pclick.client import Client
from pclick.server import Server
from pclick.Messaging_pb2 import HandshakeMessageType, HandshakeInitMessageType, SensorMessageType, ControlMessageType, ResetMessageType, ErrorMessageType, SensorRequestMessageType
from pclick.Messaging_pb2 import Message, HandshakeMessage, HandshakeInitMessage, SensorMessage, ControlMessage, ResetMessage, ErrorMessage, SensorRequestMessage
from pclick.Messaging_pb2 import ValueType
from pclick.message_proto_helpers import MessageFactory, MessageSerializer



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: Messaging.proto
# Protobuf Python Version: 5.27.0
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    5,
    27,
    0,
    '',
    'Messaging.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x0fMessaging.proto\x12\x0e\x63lick.protobuf\"G\n\x07Message\x12\x30\n\x0bmessageType\x18\x01 \x01(\x0e\x32\x1b.click.protobuf.MessageType\x12\n\n\x02id\x18\x02 \x01(\x05\"r\n\x14HandshakeInitMessage\x12\x30\n\x0bmessageType\x18\x01 \x01(\x0e\x32\x1b.click.protobuf.MessageType\x12(\n\x07version\x18\x02 \x01(\x0e\x32\x17.click.protobuf.Version\"H\n\x14SensorRequestMessage\x12\x30\n\x0bmessageType\x18\x01 \x01(\x0e\x32\x1b.click.protobuf.MessageType\"\xff\x07\n\x10HandshakeMessage\x12\x30\n\x0bmessageType\x18\x01 \x01(\x0e\x32\x1b.click.protobuf.MessageType\x12(\n\x07version\x18\x02 \x01(\x0e\x32\x17.click.protobuf.Version\x12.\n\x0b\x63ontrolType\x18\x03 \x01(\x0e\x32\x19.click.protobuf.ValueType\x12>\n\x07objects\x18\x04 \x03(\x0b\x32-.click.protobuf.HandshakeMessage.ObjectsEntry\x12H\n\x0bsimSettings\x18\x05 \x01(\x0b\x32\x33.click.protobuf.HandshakeMessage.SimulationSettings\x1a\x33\n\x07Sensors\x12(\n\x05types\x18\x01 \x03(\x0e\x32\x19.click.protobuf.ValueType\x1a\x9e\x04\n\x06Object\x12\x17\n\x0f\x63ontrolsInOrder\x18\x01 \x03(\t\x12\x36\n\x13\x63ontrolTypesInOrder\x18\x08 \x03(\x0e\x32\x19.click.protobuf.ValueType\x12\x1b\n\x13jointSensorsInOrder\x18\x07 \x03(\t\x12/\n\x0cjointSensors\x18\x02 \x03(\x0e\x32\x19.click.protobuf.ValueType\x12Q\n\rcontrolEvents\x18\x03 \x03(\x0b\x32:.click.protobuf.HandshakeMessage.Object.ControlEventsEntry\x12\x45\n\x07sensors\x18\x04 \x03(\x0b\x32\x34.click.protobuf.HandshakeMessage.Object.SensorsEntry\x12\x30\n\robjectSensors\x18\x06 \x03(\x0e\x32\x19.click.protobuf.ValueType\x1aO\n\x12\x43ontrolEventsEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12(\n\x05value\x18\x02 \x01(\x0e\x32\x19.click.protobuf.ValueType:\x02\x38\x01\x1aX\n\x0cSensorsEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\x37\n\x05value\x18\x02 \x01(\x0b\x32(.click.protobuf.HandshakeMessage.Sensors:\x02\x38\x01\x1a&\n\x12SimulationSettings\x12\x10\n\x08timeStep\x18\x01 \x01(\x01\x1aW\n\x0cObjectsEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\x36\n\x05value\x18\x02 \x01(\x0b\x32\'.click.protobuf.HandshakeMessage.Object:\x02\x38\x01\"\xb5\x03\n\x0e\x43ontrolMessage\x12\x30\n\x0bmessageType\x18\x01 \x01(\x0e\x32\x1b.click.protobuf.MessageType\x12<\n\x07objects\x18\x02 \x03(\x0b\x32+.click.protobuf.ControlMessage.ObjectsEntry\x1a\xdb\x01\n\x06Object\x12\x0e\n\x06\x61ngles\x18\x01 \x03(\x01\x12\x19\n\x11\x61ngularVelocities\x18\x02 \x03(\x01\x12\x0f\n\x07torques\x18\x03 \x03(\x01\x12\x0e\n\x06values\x18\x05 \x03(\x01\x12O\n\rcontrolEvents\x18\x04 \x03(\x0b\x32\x38.click.protobuf.ControlMessage.Object.ControlEventsEntry\x1a\x34\n\x12\x43ontrolEventsEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\x08:\x02\x38\x01\x1aU\n\x0cObjectsEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\x34\n\x05value\x18\x02 \x01(\x0b\x32%.click.protobuf.ControlMessage.Object:\x02\x38\x01\"\xdc\n\n\rSensorMessage\x12\x30\n\x0bmessageType\x18\x01 \x01(\x0e\x32\x1b.click.protobuf.MessageType\x12;\n\x07objects\x18\x03 \x03(\x0b\x32*.click.protobuf.SensorMessage.ObjectsEntry\x12\x42\n\x07simVars\x18\x04 \x01(\x0b\x32\x31.click.protobuf.SensorMessage.SimulationVariables\x1a\x13\n\x04Vec3\x12\x0b\n\x03\x61rr\x18\x01 \x03(\x01\x1a\xd3\x04\n\x06Sensor\x12\x0f\n\x05\x61ngle\x18\x01 \x01(\x01H\x00\x12\x1b\n\x11\x61ngularVelocity1d\x18\x02 \x01(\x01H\x00\x12\x12\n\x08torque1d\x18\x03 \x01(\x01H\x00\x12\x36\n\x08position\x18\x04 \x01(\x0b\x32\".click.protobuf.SensorMessage.Vec3H\x00\x12\x31\n\x03rpy\x18\x05 \x01(\x0b\x32\".click.protobuf.SensorMessage.Vec3H\x00\x12\x13\n\tactivated\x18\x06 \x01(\x08H\x00\x12<\n\x0e\x61\x63\x63\x65leration3d\x18\x07 \x01(\x0b\x32\".click.protobuf.SensorMessage.Vec3H\x00\x12\x35\n\x07\x66orce3d\x18\x08 \x01(\x0b\x32\".click.protobuf.SensorMessage.Vec3H\x00\x12\x36\n\x08torque3d\x18\t \x01(\x0b\x32\".click.protobuf.SensorMessage.Vec3H\x00\x12\x43\n\x15\x61ngularAcceleration3d\x18\n \x01(\x0b\x32\".click.protobuf.SensorMessage.Vec3H\x00\x12\x38\n\nvelocity3d\x18\x0c \x01(\x0b\x32\".click.protobuf.SensorMessage.Vec3H\x00\x12?\n\x11\x61ngularVelocity3d\x18\r \x01(\x0b\x32\".click.protobuf.SensorMessage.Vec3H\x00\x12\x11\n\x07\x66orce1d\x18\x0e \x01(\x01H\x00\x42\x07\n\x05value\x1a?\n\x07Sensors\x12\x34\n\x06sensor\x18\x01 \x03(\x0b\x32$.click.protobuf.SensorMessage.Sensor\x1a\xe7\x02\n\x06Object\x12\x14\n\x0c\x61ngleSensors\x18\x01 \x03(\x01\x12\x1e\n\x16\x61ngularVelocitySensors\x18\x02 \x03(\x01\x12\x15\n\rtorqueSensors\x18\x03 \x03(\x01\x12\x38\n\x0c\x66orceSensors\x18\x05 \x03(\x0b\x32\".click.protobuf.SensorMessage.Vec3\x12;\n\robjectSensors\x18\x04 \x03(\x0b\x32$.click.protobuf.SensorMessage.Sensor\x12\x42\n\x07sensors\x18\x06 \x03(\x0b\x32\x31.click.protobuf.SensorMessage.Object.SensorsEntry\x1aU\n\x0cSensorsEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\x34\n\x05value\x18\x02 \x01(\x0b\x32%.click.protobuf.SensorMessage.Sensors:\x02\x38\x01\x1a,\n\x13SimulationVariables\x12\x15\n\rsimulatedTime\x18\x01 \x01(\x01\x1aT\n\x0cObjectsEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\x33\n\x05value\x18\x02 \x01(\x0b\x32$.click.protobuf.SensorMessage.Object:\x02\x38\x01\"@\n\x0cResetMessage\x12\x30\n\x0bmessageType\x18\x01 \x01(\x0e\x32\x1b.click.protobuf.MessageType\"\x80\x01\n\x0c\x45rrorMessage\x12\x30\n\x0bmessageType\x18\x01 \x01(\x0e\x32\x1b.click.protobuf.MessageType\x12(\n\x05\x65rror\x18\x02 \x01(\x0e\x32\x19.click.protobuf.ErrorType\x12\x14\n\x0c\x65rrormessage\x18\x03 \x01(\t*\xbe\x01\n\x0bMessageType\x12\x1c\n\x18HandshakeInitMessageType\x10\x00\x12\x18\n\x14HandshakeMessageType\x10\x01\x12\x1c\n\x18SensorRequestMessageType\x10\x06\x12\x16\n\x12\x43ontrolMessageType\x10\x02\x12\x15\n\x11SensorMessageType\x10\x03\x12\x14\n\x10ResetMessageType\x10\x04\x12\x14\n\x10\x45rrorMessageType\x10\x05*\xed\x01\n\tValueType\x12\t\n\x05\x41ngle\x10\x00\x12\x15\n\x11\x41ngularVelocity1D\x10\x01\x12\x0c\n\x08Torque1D\x10\x02\x12\x0c\n\x08Position\x10\x03\x12\x07\n\x03RPY\x10\x04\x12\r\n\tActivated\x10\x05\x12\x12\n\x0e\x41\x63\x63\x65leration3D\x10\x06\x12\x0b\n\x07\x46orce1D\x10\x0c\x12\x0b\n\x07\x46orce3D\x10\x07\x12\x0c\n\x08Torque3D\x10\x08\x12\x19\n\x15\x41ngularAcceleration3D\x10\t\x12\x0e\n\nVelocity3D\x10\n\x12\x15\n\x11\x41ngularVelocity3D\x10\x0b\x12\x0c\n\x08Multiple\x10\x0f*-\n\x07Version\x12\r\n\tUNDEFINED\x10\x00\x12\x13\n\x0f\x43URRENT_VERSION\x10\x01*$\n\tErrorType\x12\x17\n\x13VersionNotSupported\x10\x00\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'Messaging_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  DESCRIPTOR._loaded_options = None
  _globals['_HANDSHAKEMESSAGE_OBJECT_CONTROLEVENTSENTRY']._loaded_options = None
  _globals['_HANDSHAKEMESSAGE_OBJECT_CONTROLEVENTSENTRY']._serialized_options = b'8\001'
  _globals['_HANDSHAKEMESSAGE_OBJECT_SENSORSENTRY']._loaded_options = None
  _globals['_HANDSHAKEMESSAGE_OBJECT_SENSORSENTRY']._serialized_options = b'8\001'
  _globals['_HANDSHAKEMESSAGE_OBJECTSENTRY']._loaded_options = None
  _globals['_HANDSHAKEMESSAGE_OBJECTSENTRY']._serialized_options = b'8\001'
  _globals['_CONTROLMESSAGE_OBJECT_CONTROLEVENTSENTRY']._loaded_options = None
  _globals['_CONTROLMESSAGE_OBJECT_CONTROLEVENTSENTRY']._serialized_options = b'8\001'
  _globals['_CONTROLMESSAGE_OBJECTSENTRY']._loaded_options = None
  _globals['_CONTROLMESSAGE_OBJECTSENTRY']._serialized_options = b'8\001'
  _globals['_SENSORMESSAGE_OBJECT_SENSORSENTRY']._loaded_options = None
  _globals['_SENSORMESSAGE_OBJECT_SENSORSENTRY']._serialized_options = b'8\001'
  _globals['_SENSORMESSAGE_OBJECTSENTRY']._loaded_options = None
  _globals['_SENSORMESSAGE_OBJECTSENTRY']._serialized_options = b'8\001'
  _globals['_MESSAGETYPE']._serialized_start=3337
  _globals['_MESSAGETYPE']._serialized_end=3527
  _globals['_VALUETYPE']._serialized_start=3530
  _globals['_VALUETYPE']._serialized_end=3767
  _globals['_VERSION']._serialized_start=3769
  _globals['_VERSION']._serialized_end=3814
  _globals['_ERRORTYPE']._serialized_start=3816
  _globals['_ERRORTYPE']._serialized_end=3852
  _globals['_MESSAGE']._serialized_start=35
  _globals['_MESSAGE']._serialized_end=106
  _globals['_HANDSHAKEINITMESSAGE']._serialized_start=108
  _globals['_HANDSHAKEINITMESSAGE']._serialized_end=222
  _globals['_SENSORREQUESTMESSAGE']._serialized_start=224
  _globals['_SENSORREQUESTMESSAGE']._serialized_end=296
  _globals['_HANDSHAKEMESSAGE']._serialized_start=299
  _globals['_HANDSHAKEMESSAGE']._serialized_end=1322
  _globals['_HANDSHAKEMESSAGE_SENSORS']._serialized_start=597
  _globals['_HANDSHAKEMESSAGE_SENSORS']._serialized_end=648
  _globals['_HANDSHAKEMESSAGE_OBJECT']._serialized_start=651
  _globals['_HANDSHAKEMESSAGE_OBJECT']._serialized_end=1193
  _globals['_HANDSHAKEMESSAGE_OBJECT_CONTROLEVENTSENTRY']._serialized_start=1024
  _globals['_HANDSHAKEMESSAGE_OBJECT_CONTROLEVENTSENTRY']._serialized_end=1103
  _globals['_HANDSHAKEMESSAGE_OBJECT_SENSORSENTRY']._serialized_start=1105
  _globals['_HANDSHAKEMESSAGE_OBJECT_SENSORSENTRY']._serialized_end=1193
  _globals['_HANDSHAKEMESSAGE_SIMULATIONSETTINGS']._serialized_start=1195
  _globals['_HANDSHAKEMESSAGE_SIMULATIONSETTINGS']._serialized_end=1233
  _globals['_HANDSHAKEMESSAGE_OBJECTSENTRY']._serialized_start=1235
  _globals['_HANDSHAKEMESSAGE_OBJECTSENTRY']._serialized_end=1322
  _globals['_CONTROLMESSAGE']._serialized_start=1325
  _globals['_CONTROLMESSAGE']._serialized_end=1762
  _globals['_CONTROLMESSAGE_OBJECT']._serialized_start=1456
  _globals['_CONTROLMESSAGE_OBJECT']._serialized_end=1675
  _globals['_CONTROLMESSAGE_OBJECT_CONTROLEVENTSENTRY']._serialized_start=1623
  _globals['_CONTROLMESSAGE_OBJECT_CONTROLEVENTSENTRY']._serialized_end=1675
  _globals['_CONTROLMESSAGE_OBJECTSENTRY']._serialized_start=1677
  _globals['_CONTROLMESSAGE_OBJECTSENTRY']._serialized_end=1762
  _globals['_SENSORMESSAGE']._serialized_start=1765
  _globals['_SENSORMESSAGE']._serialized_end=3137
  _globals['_SENSORMESSAGE_VEC3']._serialized_start=1961
  _globals['_SENSORMESSAGE_VEC3']._serialized_end=1980
  _globals['_SENSORMESSAGE_SENSOR']._serialized_start=1983
  _globals['_SENSORMESSAGE_SENSOR']._serialized_end=2578
  _globals['_SENSORMESSAGE_SENSORS']._serialized_start=2580
  _globals['_SENSORMESSAGE_SENSORS']._serialized_end=2643
  _globals['_SENSORMESSAGE_OBJECT']._serialized_start=2646
  _globals['_SENSORMESSAGE_OBJECT']._serialized_end=3005
  _globals['_SENSORMESSAGE_OBJECT_SENSORSENTRY']._serialized_start=2920
  _globals['_SENSORMESSAGE_OBJECT_SENSORSENTRY']._serialized_end=3005
  _globals['_SENSORMESSAGE_SIMULATIONVARIABLES']._serialized_start=3007
  _globals['_SENSORMESSAGE_SIMULATIONVARIABLES']._serialized_end=3051
  _globals['_SENSORMESSAGE_OBJECTSENTRY']._serialized_start=3053
  _globals['_SENSORMESSAGE_OBJECTSENTRY']._serialized_end=3137
  _globals['_RESETMESSAGE']._serialized_start=3139
  _globals['_RESETMESSAGE']._serialized_end=3203
  _globals['_ERRORMESSAGE']._serialized_start=3206
  _globals['_ERRORMESSAGE']._serialized_end=3334
# @@protoc_insertion_point(module_scope)



import zmq
from pclick.message_proto_helpers import MessageSerializer, Message


class Client:
    def __init__(self):
        self.context = zmq.Context()
        self.socket = self.context.socket(zmq.REQ)

    def connect(self, addr: str):
        """
        Asynchronously connects to an endpoint.
        Happily keep trying to connect until there is something there.
        """
        self.socket.connect(addr)

    def recv(self, block=True) -> Message:
        """ Receive a Message. Returns Message, or None if block parameter is True and no Message is waiting.

        block -- whether to wait for message or return directly (default True)
        """
        if block:
            return MessageSerializer.from_bytes(self.socket.recv())
        try:
            return MessageSerializer.from_bytes(self.socket.recv(flags=zmq.NOBLOCK))
        except zmq.Again as e:
            return None

    def send(self, message: Message):
        """ Send a message """
        return self.socket.send(message.SerializeToString())

    def stop(self):
        self.context.destroy()



#
#   Click server in Python
#   Binds REP socket to tcp://*:5555
#   Expects Message from client, replies with Message
#

import os
import signal
from argparse import ArgumentParser
from pclick import Server, MessageFactory, ControlMessageType, HandshakeInitMessageType, ValueType
from pclick.server import SizeCollectorChanges


def parse_args():
    parser = ArgumentParser(description='Demo client connecting to click server')
    parser.add_argument('--host', metavar='<host>', type=str, default="*",
                        help=f'server to connect to, default is *')
    parser.add_argument('--port', metavar='<port>', type=str, default="5555",
                        help=f'port to connect to, default is 5555')
    parser.add_argument('--addr', metavar='<addr>', type=str, default="",
                        help=f'set addr. Ie ipc:///tmp/click.ipc. host and port will be ignored')
    parser.add_argument('--trace', action='store_true',
                        help=f'print what is sent/received')
    parser.add_argument('--trace-sizes', action='store_true',
                        help=f'print size of what is sent/received')
    return parser.parse_args()

def handler(signum, frame):
    os._exit(0)

def main():
    signal.signal(signal.SIGINT, handler)
    args = parse_args()
    addr = f"tcp://{args.host}:{args.port}"
    if args.addr:
        addr = args.addr
    server = Server(addr)
    if args.trace_sizes:
        server.size_collector = SizeCollectorChanges()

    # Note: Below code uses the protobuf API directly, we recommend using the higher level agxclick ClickObject and ClickRobot instead
    # if possible - to protect client code from future protocol changes.
    while True:

        request = server.recv()
        if request is None:
            continue
        if args.trace:
            print(f"Received request: {request}")

        if request.messageType == HandshakeInitMessageType:
            response = handshake_message()
            if args.trace:
                print(f"Sending handshake: {response}")
            server.send(response)
        if request.messageType == ControlMessageType:
            response = sensor_message()
            if args.trace:
                print(f"Sending sensormessage: {response}")
            server.send(response)
        if args.trace_sizes and server.size_collector.is_updated:
            print(f"Received {server.size_collector.recv_size} from client, Sent {server.size_collector.send_size} bytes to client")


def handshake_message():
    handshake = MessageFactory.create_handshake()
    handshake.controlType = ValueType.Force3D
    object = handshake.objects["robot"]

    object.controlsInOrder.extend(["joint1", "joint2"])
    object.controlTypesInOrder.extend([ValueType.Angle, ValueType.Angle])
    object.jointSensorsInOrder.extend(["joint1", "joint2"])
    object.controlEvents["gripper"] = ValueType.Activated
    object.jointSensors.extend([ValueType.Angle, ValueType.AngularVelocity1D, ValueType.Torque1D])
    object.objectSensors.append(ValueType.Position)
    object.sensors["external_1"].types.extend([ValueType.Force3D, ValueType.AngularAcceleration3D])
    return handshake


def sensor_message():
    sensor_m = MessageFactory.create_sensormessage()
    robot = sensor_m.objects["robot1"]

    size = 2
    robot.angleSensors.extend([1.0] * size)
    robot.angularVelocitySensors.extend([2.0] * size)
    robot.torqueSensors.extend([3.0] * size)

    box = sensor_m.objects["box"]
    sensor = box.objectSensors.add()
    sensor.position.arr.extend([1.0, 2.0, 3.0])
    sensor = box.objectSensors.add()
    sensor.rpy.arr.extend([4.0, 5.0, 6.0])
    val = robot.sensors["external_1"].sensor.add()
    val.force3d.arr.extend([4.0, 4.1, 4.2])
    val = robot.sensors["external_1"].sensor.add()
    val.angularAcceleration3d.arr.extend([5.0, 5.1, 5.2])

    return sensor_m


main()



#
#   click client in Python
#   Connects REQ socket to tcp://localhost:5555
#   Sends "HandshakeInit" to server, expects "Handshake" back
#

from argparse import ArgumentParser
from time import time
import os
import signal
from pclick import Client
from pclick import MessageFactory
from pclick.Messaging_pb2 import SensorMessageType, ResetMessageType

def parse_args():
    parser = ArgumentParser(description='Demo client connecting to click server')
    parser.add_argument('--host', metavar='<host>', type=str, default="localhost",
                        help=f'server to connect to, default is localhost')
    parser.add_argument('--port', metavar='<port>', type=str, default="5555",
                        help=f'port to connect to, default is 5555')
    parser.add_argument('--addr', metavar='<addr>', type=str, default="",
                        help=f'set addr. Ie ipc:///tmp/click.ipc. host and port will be ignored')
    parser.add_argument('--range', metavar='<range>', type=int, default=0,
                        help=f'How many extra messages to send/recv, default is 0')
    parser.add_argument('--handshake-init', dest='handshake_init', action="store_true", default=True,
                        help=f'Send handshake init. This is the default')
    parser.add_argument('--sensorrequest', dest='sensorrequest', action="store_true",
                        help=f'Send sensor request message')
    parser.add_argument('--controlmessage', metavar='<value>', type=str, default=None,
                        help=f'send controlmessage with control values set to <value>. Overrides --handshake-init. Examples --controlmessage robot1:0,0;panda2:1,1')
    parser.add_argument('--controltype', metavar='<value>', choices=["Angle", "AngularVelocity1D", "Torque1D"], default=None,
                        help=f'How to interpret controlmessage values, ie Angle(radians), AngularVelocity(radians/sec), or Torque(Nm). Default is to interpret each value individually')
    parser.add_argument('--errormessage', dest='errormessage', action="store_true",
                        help=f'Send error message')
    parser.add_argument('--resetmessage', dest='resetmessage', action="store_true",
                        help=f'Send reset message')
    parser.add_argument('--end-with-errormessage', dest='end_with_errormessage', action="store_true",
                        help=f'Send an extra error message before quitting')
    return parser.parse_args()


def send_errormessage(client: Client):
    message = MessageFactory.create_errormessage()
    print("Sending errormessage")
    client.send(message)

def handler(signum, frame):
    os._exit(0)

signal.signal(signal.SIGINT, handler)
args = parse_args()
addr = f"tcp://{args.host}:{args.port}"
if args.addr:
    addr = args.addr

#  Socket to talk to server
client = Client()
print(f"Connecting to click server {addr}")
client.connect(addr)

start_time = time()

if args.controlmessage:
    message = MessageFactory.create_controlmessage()
    # Note: Below code uses the protobuf API directly, which does not protect client code from future protocol changes.
    # The C++ Click API has a client layer adding that protection that has not been implemented in python yet.
    robotargs = args.controlmessage.split(";")
    for arg in robotargs:
        robotname, *values = arg.split(":")
        robot = message.objects[robotname]
        arr = list(map(float, values[0].split(",")))
        if args.controltype is None:
            robot.values.extend(arr)
        elif args.controltype.lower() == "angle":
            robot.angles.extend(arr)
        elif args.controltype.lower() == "angularvelocity1D":
            robot.angularVelocities.extend(arr)
        elif args.controltype.lower() == "torque1d":
            robot.torques.extend(arr)
    print(f"Sending {str(message)}")
    client.send(message)
elif args.resetmessage:
    message = MessageFactory.create_resetmessage()
    print("Sending resetmessage")
    client.send(message)
elif args.sensorrequest:
    message = MessageFactory.create_sensorrequestmessage()
    print("Sending sensor request message")
    client.send(message)
elif args.errormessage:
    send_errormessage(client)
    client.stop()
    os._exit(0)
else:
    message = MessageFactory.create_handshake_init()
    print("Sending initiate handshake")
    client.send(message)

response = client.recv()
print(f"Received response {response}")

for i in range(0, args.range):
    client.send(message)
    response = client.recv()
    if (response.messageType == ResetMessageType):
        print("Received ResetMessage")

wall_time = time() - start_time
freq = (args.range + 1) / wall_time
print(f"Sent/received {args.range + 1} messages in {wall_time:.2f} seconds, ie {freq:.0f} Hz")

if args.end_with_errormessage:
    send_errormessage(client)



import zmq
from pclick.message_proto_helpers import MessageSerializer, Message
from abc import abstractmethod


class SizeCollector:
    """
    A no operation SizeCollector.
    """

    def sendSize(self, len: int):
        pass

    def recvSize(self, len: int):
        pass


class SizeCollectorChanges(SizeCollector):
    """
    A SizeCollector that enables retrieving changes in send/receive sizes.
    is_updated() returns True if there is a change.
    """
    send_updated = False
    recv_updated = False
    _send_size = None
    _recv_size = None

    @property
    def is_updated(self):
        """
        Returns True if there is a change in send_size or recv_size.
        Side effect: Resets change, i.e. calling is_updated() twice after change will yield True, then False.
        """
        res = self.recv_updated or self.send_updated
        self.send_updated = self.recv_updated = False
        return res

    @property
    def send_size(self):
        return self._send_size

    @property
    def recv_size(self):
        return self._recv_size

    def sendSize(self, len: int):
        assert len > 0
        self.send_updated = len != self._send_size
        self._send_size = len

    def recvSize(self, len: int):
        assert len > 0
        self.recv_updated = len != self._recv_size
        self._recv_size = len


class Server:
    def __init__(self, addr: str, size_collector=SizeCollector()):
        self.context = zmq.Context()
        self.socket = self.context.socket(zmq.REP)
        self.addr = addr
        self.socket.bind(self.addr)
        self.size_collector = size_collector

    def recv(self) -> Message:
        """
        Non-blocking receive. Returns Message or None if no Message is waiting.
        """
        try:
            serialized = self.socket.recv(flags=zmq.NOBLOCK)
            self.size_collector.recvSize(len(serialized))
            return MessageSerializer.from_bytes(serialized)

        except zmq.Again as e:
            return None

    def recv_blocking(self) -> Message:
        """
        Blocking receive
        """
        return MessageSerializer.from_bytes(self.socket.recv())

    def send(self, message: Message):
        """
        Non-blocking send. Throws Exception if buffer is full
        """
        serialized = message.SerializeToString()
        self.size_collector.sendSize(len(serialized))
        return self.socket.send(serialized, flags=zmq.NOBLOCK)

    def stop(self):
        self.socket.close()
        self.context.destroy()



from pclick.Messaging_pb2 import CURRENT_VERSION, Message, ErrorMessage, HandshakeMessage, SensorMessage, ControlMessage, ResetMessage, HandshakeInitMessage, SensorRequestMessage
from pclick.Messaging_pb2 import HandshakeInitMessageType, ErrorMessageType, HandshakeMessageType, SensorMessageType, ControlMessageType, ResetMessageType, SensorRequestMessageType


class MessageFactory:
    _mdict = {HandshakeInitMessageType: HandshakeInitMessage,
              HandshakeMessageType: HandshakeMessage,
              ControlMessageType: ControlMessage,
              SensorMessageType: SensorMessage,
              ResetMessageType: ResetMessage,
              ErrorMessageType: ErrorMessage,
              SensorRequestMessageType: SensorRequestMessage}

    @classmethod
    def _create_message(cls, mtype):
        message = cls._mdict[mtype]()
        if hasattr(message, "version"):
            message.version = CURRENT_VERSION
        message.messageType = mtype
        return message

    @classmethod
    def create_handshake_init(cls):
        return cls._create_message(HandshakeInitMessageType)

    @classmethod
    def create_handshake(cls):
        return cls._create_message(HandshakeMessageType)

    @classmethod
    def create_controlmessage(cls):
        return cls._create_message(ControlMessageType)

    @classmethod
    def create_sensormessage(cls):
        return cls._create_message(SensorMessageType)

    @classmethod
    def create_resetmessage(cls):
        return cls._create_message(ResetMessageType)

    @classmethod
    def create_errormessage(cls):
        return cls._create_message(ErrorMessageType)

    @classmethod
    def create_sensorrequestmessage(cls):
        return cls._create_message(SensorRequestMessageType)


class MessageSerializer:
    @classmethod
    def from_bytes(cls, bytes: str):
        message = Message()
        message.ParseFromString(bytes)

        m = MessageFactory._mdict[message.messageType]()
        m.ParseFromString(bytes)
        return m
